@@ -10,20 +10,22 @@ import os
 import re
 import shutil
 import sys
+import weakref
+import zipfile
     
 from distutils.spawn import find_executable
 from functools import wraps
 from tempfile import _TemporaryFileWrapper as TemporaryFileWrapperBase
 
-from clu.constants.consts import DELETE_FLAG, ENCODING, PATH, SCRIPT_PATH
+from clu.constants.consts import λ, DELETE_FLAG, ENCODING, PATH, SCRIPT_PATH
 from clu.constants.exceptions import ExecutionError, FilesystemError
 from clu.constants.polyfills import lru_cache, scandir, walk
 from clu.predicates import attr, allattrs, anyof
 from clu.sanitizer import utf8_encode
 from clu.typology import ispath, isvalidpath
-from .misc import masked_permissions
+from .misc import filesize, gethomedir, masked_permissions
 from .misc import stringify, suffix_searcher, swapext, u8str
-from clu.exporting import Exporter
+from clu.exporting import ValueDescriptor, Exporter
 
 exporter = Exporter(path=__file__)
 export = exporter.decorator()
@@ -260,8 +262,8 @@ class TypeLocker(abc.ABCMeta):
         
         … The point of this is to allow any of the classes throughout the
         clu.fs.filesystem module regardless of where they are defined
-        or from whom they inherit, to make use of cheaply-constructed Directory
-        instances wherever convenient.
+        or from whom they inherit, to make use of cheaply-constructed
+        Directory instances wherever convenient.
         
         Because the “directory(…)” method installed by TypeLocker performs
         a lazy-lookup of the Directory class, using its own type index dict,
@@ -271,30 +273,41 @@ class TypeLocker(abc.ABCMeta):
         is utilized within at least one Directory method – sans any issues.
     """
     
-    # The metaclass-internal dictionary of generated classes:
-    types = collections.OrderedDict()
-    
-    @classmethod
-    def __prepare__(metacls, name, bases, **kwargs):
-        """ Maintain declaration order in class members: """
-        return collections.OrderedDict()
+    # The metaclass-internal dictionary of descendant type weakrefs:
+    types = weakref.WeakValueDictionary()
     
     def __new__(metacls, name, bases, attributes, **kwargs):
         """ All classes are initialized with a “directory(…)”
-            class method, lazily returning an instance of the
-            clu.fs.filesystem.Directory(…) class, per
-            the arguments:
+            static method, lazily returning an instance of the
+            clu.fs.filesystem.Directory(…) class.
+            
+            A read-only descriptor shadows the “types” attribute,
+            to block access to the metaclass type-registry dict
+            from generated subtypes, as well.
         """
+        # Fill in the “types” attribute to prevent the metaclass’
+        # registry dict from leaking into subtypes:
+        attributes['types']         = ValueDescriptor(tuple())
+        
         # Always replace the “directory” method anew:
-        attributes['directory'] = staticmethod(
-                                      lambda pth=None: \
-                                      metacls.types['Directory'](pth=pth))
+        directory = lambda pth=None: metacls.types['Directory'](pth=pth)
+        
+        directory.__name__          = 'directory'
+        directory.__qualname__      = f'{name}.directory'
+        directory.__lambda_name__   = λ
+        attributes['directory']     = staticmethod(directory)
+        
+        # Call up (using a vanilla attributes dict):
         cls = super(TypeLocker, metacls).__new__(metacls, name,
                                                           bases,
-                                                          dict(attributes),
+                                                          attributes,
                                                         **kwargs)
+        
+        # Register with clu.fs.TypeLocker and os.PathLike:
         metacls.types[name] = cls
         os.PathLike.register(cls)
+        
+        # Return the new type
         return cls
 
 class TemporaryFileWrapper(TemporaryFileWrapperBase,
@@ -323,26 +336,28 @@ class TemporaryFileWrapper(TemporaryFileWrapperBase,
 cache = lru_cache(maxsize=128, typed=True)
 
 @cache
-def TemporaryNamedFile(tempth, mode='wb', buffer_size=-1, delete=True):
+def TemporaryNamedFile(temppath, mode='wb',
+                                 delete=True,
+                                 buffer_size=-1):
     """ Variation on ``tempfile.NamedTemporaryFile(…)``, for use within
         `filesystem.TemporaryName()` – q.v. class definition sub.
         
         Parameters
         ----------
-        tempth : str / bytes  / filename-ish
+        temppath : str / bytes  / filename-ish
             File name, path, or filename-ish instance to open.
         mode : str / bytes, optional
             String-like symbolic explication of mode with which to open
             the file -- q.v. ``io.open(…)`` or ``__builtins__.open(…)``
             supra.
-        buffer_size : int, optional
-            Integer indicating buffer size to use during file reading
-            and/or writing. Default value is -1 (which indicates that
-            reads and writes should be unbuffered).
         delete : bool, optional
             Boolean value indicating whether to delete the wrapped
             file upon scope exit or interpreter shutdown (whichever
             happens first). Default is True.
+        buffer_size : int, optional
+            Integer indicating buffer size to use during file reading
+            and/or writing. Default value is -1 (which indicates that
+            reads and writes should be unbuffered).
         
         Returns
         -------
@@ -372,7 +387,7 @@ def TemporaryNamedFile(tempth, mode='wb', buffer_size=-1, delete=True):
     path = None
     
     try:
-        path = os.fspath(tempth)
+        path = os.fspath(temppath)
         descriptor = os.open(path, flags)
         filehandle = os.fdopen(descriptor, mode, buffer_size)
         return TemporaryFileWrapper(filehandle, path, delete)
@@ -397,10 +412,10 @@ class TemporaryName(collections.abc.Hashable,
         Unless you say not to. Really it's your call dogg I could give AF
     """
     
-    fields = ('name', 'exists', 'destroy',
-              'mode', 'binary_mode',
-              'prefix', 'suffix',
-              'parent')
+    fields = ('name',   'exists', 
+                        'destroy',
+              'mode',   'binary_mode',
+              'prefix', 'suffix', 'parent')
     
     def __init__(self, prefix=None, suffix="tmp",
                        parent=None,
@@ -539,8 +554,13 @@ class TemporaryName(collections.abc.Hashable,
             on the filehandle instance you use. That’s fair, right? I think that’s
             totally fair to do it like that, OK.
         """
-        return TemporaryNamedFile(self.do_not_destroy(),
-                                  mode=self.mode)
+        return TemporaryNamedFile(temppath=self.do_not_destroy(),
+                                      mode=self.mode)
+    
+    @property
+    def filesize(self):
+        """ The filesize for the temporary file """
+        return filesize(self._name)
     
     def split(self):
         """ Return (dirname, basename) e.g. for /yo/dogg/i/heard/youlike,
@@ -566,6 +586,29 @@ class TemporaryName(collections.abc.Hashable,
                                 follow_symlinks=True))
         return False
     
+    def read(self, *, original_position=False):
+        """ Read data from the temporary name, if it points to an existing file """
+        orig, out = 0, b""
+        if not self.exists:
+            if original_position:
+                return orig, out
+            return out
+        with open(self._name, "r+b") as handle:
+            orig = handle.seek(0)
+            out += handle.read()
+        if original_position:
+            return orig, out
+        return out
+    
+    def write(self, data):
+        """ Write data to the temporary name using a context-managed handle """
+        ensure_path_is_valid(self._name)
+        bytestring = utf8_encode(data)
+        with open(self._name, "xb") as handle:
+            handle.write(bytestring)
+            handle.flush()
+        return self.exists
+    
     def do_not_destroy(self):
         """ Mark this TemporaryName instance as one that should not be automatically
             destroyed upon the scope exit for the instance.
@@ -853,13 +896,13 @@ class Directory(collections.abc.Hashable,
         self.ctx_initialize()
         return False
     
-    def realpath(self, pth=None):
+    def realpath(self, source=None):
         """ Sugar for calling os.path.realpath(self.name) """
         return u8str(
             os.path.realpath(
-            os.fspath(pth or self.name)))
+            os.fspath(source or self.name)))
     
-    def ls(self, pth=None, suffix=None):
+    def ls(self, suffix=None, source=None):
         """ List files -- defaults to the process’ current working directory.
             As per the UNIX custom, files whose name begins with a dot are
             omitted.
@@ -869,9 +912,9 @@ class Directory(collections.abc.Hashable,
         """
         return tuple(filter(suffix_searcher(suffix),
                     (direntry.name for direntry in filter(non_dotfile_matcher,
-                                                  scandir(self.realpath(pth))))))
+                                                  scandir(self.realpath(source))))))
     
-    def ls_la(self, pth=None, suffix=None):
+    def ls_la(self, suffix=None, source=None):
         """ List all files, including files whose name starts with a dot.
             The default is to use the process’ current working directory.
             
@@ -886,35 +929,35 @@ class Directory(collections.abc.Hashable,
             I think looks awkward and goofy.)
         """
         return tuple(filter(suffix_searcher(suffix),
-                    (direntry.name for direntry in scandir(self.realpath(pth)))))
+                    (direntry.name for direntry in scandir(self.realpath(source)))))
     
-    def subpath(self, subpth, whence=None, requisite=False):
+    def subpath(self, subpath, source=None, requisite=False):
         """ Returns the path to a subpath of the instances’ target path. """
-        fullpth = os.path.join(os.fspath(whence or self.name),
-                               os.fspath(subpth))
-        return (os.path.exists(fullpth) or not requisite) and fullpth or None
+        fullpath = os.path.join(os.fspath(source or self.name),
+                                os.fspath(subpath))
+        return (os.path.exists(fullpath) or not requisite) and fullpath or None
     
-    def subdirectory(self, subdir, whence=None):
+    def subdirectory(self, subdir, source=None):
         """ Returns the path to a subpath of the instances’ target path --
             much like Directory.subpath(…) -- as an instance of Directory.
         """
-        pth = self.subpath(subdir, whence, requisite=False)
-        if os.path.isfile(pth):
-            raise FilesystemError(f"file exists at subdirectory path: {pth}")
-        if os.path.islink(pth):
-            raise FilesystemError(f"symlink exists at subdirectory path: {pth}")
-        if os.path.ismount(pth):
-            raise FilesystemError(f"mountpoint exists at subdirectory path: {pth}")
-        return self.directory(pth)
+        path = self.subpath(subdir, source, requisite=False)
+        if os.path.isfile(path):
+            raise FilesystemError(f"file exists at subdirectory path: {path}")
+        if os.path.islink(path):
+            raise FilesystemError(f"symlink exists at subdirectory path: {path}")
+        if os.path.ismount(path):
+            raise FilesystemError(f"mountpoint exists at subdirectory path: {path}")
+        return self.directory(path)
     
-    def makedirs(self, pth=None, mode=0o755):
+    def makedirs(self, subpath=None, mode=0o755):
         """ Creates any parts of the target directory path that don’t
             already exist, á la the `mkdir -p` shell command.
         """
         try:
             os.makedirs(os.path.abspath(
                         os.path.join(self.name,
-                        os.fspath(pth or os.curdir))),
+                        os.fspath(subpath or os.curdir))),
                         exist_ok=False,
                         mode=masked_permissions(mode))
         except OSError as os_error:
@@ -1019,7 +1062,7 @@ class Directory(collections.abc.Hashable,
                     assert os.path.samefile(dstfile, outfile)
                     results.append(dstfile)
             # Return the destination directory instance and the result list:
-            return whereto, results
+            return whereto, tuple(results)
     
     def copy_all(self, destination):
         """ Copy the entire directory tree, all contents included, to a new
@@ -1053,28 +1096,27 @@ class Directory(collections.abc.Hashable,
                 f"copy_all() source doesn’t exist: {self.name}")
         return False
     
-    def zip_archive(self, zpth=None, zmode=None):
+    def zip_archive(self, destination, compression_mode=zipfile.ZIP_DEFLATED):
         """ Recursively descends through the target directory, stowing all
-            that it finds into a zipfile at the specified path.
+            that it finds into a zipfile at the specified destination path.
             
-            Use the optional “zmode” parameter to specify the compression
-            algorithm, as per the constants found in the `zipfile` module;
-            the default value is `zipfile.ZIP_DEFLATED`.
+            Use the optional “compression_mode” parameter to specify the
+            compression algorithm, as per the constants found in the `zipfile`
+            module; the default value is `zipfile.ZIP_DEFLATED`.
         """
-        import zipfile
-        if zpth is None:
-            raise FilesystemError("Need to specify a zip-archive file path")
-        zpth = os.fspath(zpth)
+        if destination is None:
+            raise FilesystemError("zip-archive destination path cannot be None")
+        zpth = os.fspath(destination)
         zsuf = type(self).zip_suffix
         if not zpth.lower().endswith(zsuf):
-            zpth += zsuf
-        if not zmode:
-            zmode = zipfile.ZIP_DEFLATED
+            zpth = swapext(zpth, zsuf)
+        if not compression_mode:
+            compression_mode = zipfile.ZIP_DEFLATED
         ensure_path_is_valid(zpth)
         with TemporaryName(prefix="ziparchive-",
                            suffix=zsuf[1:],
                            randomized=True) as ztmp:
-            with zipfile.ZipFile(ztmp.name, "w", zmode) as ziphandle:
+            with zipfile.ZipFile(ztmp.name, "w", compression_mode) as ziphandle:
                 for root, dirs, files in self.walk():
                     ziphandle.write(root, self.relparent(root)) # add directory
                     for filename in files:
@@ -1085,9 +1127,9 @@ class Directory(collections.abc.Hashable,
             assert ztmp.copy(zpth)
         return self.realpath(zpth)
     
-    def symlink(self, destination, pth=None):
+    def symlink(self, destination, source=None):
         """ Create a symlink at `destination`, pointing to this instances’
-            directory path (or an alternative path, if specified).
+            directory path (or an alternative source path, if specified).
             
             The `destination` argument can be anything path-like: instances of
             `str`, `unicode`, `bytes`, `bytearray`, `pathlib.Path`, `os.PathLike`,
@@ -1095,7 +1137,7 @@ class Directory(collections.abc.Hashable,
         """
         if destination is None:
             raise FilesystemError("symlink destination cannot be None")
-        os.symlink(os.fspath(pth or self.name),
+        os.symlink(os.fspath(source or self.name),
                    os.fspath(destination),
                    target_is_directory=True)
         return self
@@ -1196,7 +1238,7 @@ class hd(Directory):
     def __init__(self):
         """ Initialize a Directory instance for the current user’s home directory.
         """
-        super(hd, self).__init__(pth=os.path.expanduser("~"))
+        super(hd, self).__init__(pth=gethomedir())
 
 @export
 class TemporaryDirectory(Directory):
@@ -1373,9 +1415,5 @@ def NamedTemporaryFile(mode='w+b', buffer_size=-1,
             os.close(descriptor)
         raise FilesystemError(str(base_exception))
 
-# NO DOCS ALLOWED:
-export(DEFAULT_PREFIX,          name='DEFAULT_PREFIX')
-export(DEFAULT_TIMEOUT,         name='DEFAULT_TIMEOUT')
-
 # Assign the modules’ `__all__` and `__dir__` using the exporter:
 __all__, __dir__ = exporter.all_and_dir()
\ No newline at end of file
