@@ -16,6 +16,7 @@ class TestFsFilesystem(object):
     def test_flatten(self, datadir,
                            temporarydir):
         from clu.testing.utils import countfiles
+        from clu.typology import isvalidpathlist
         
         # target directory should not already exist:
         target = temporarydir.subdirectory('yodogg')
@@ -29,6 +30,9 @@ class TestFsFilesystem(object):
         # target and destination should be the same thing:
         assert target == destination
         
+        # files should be a valid path list:
+        assert isvalidpathlist(files)
+        
         # datadir, target and destination should each
         # contain the same amount of things, regardless
         # of any of their structures:
@@ -44,6 +48,7 @@ class TestFsFilesystem(object):
     def test_flatten_with_suffix_filter(self, datadir,
                                               temporarydir):
         from clu.testing.utils import countfiles
+        from clu.typology import isvalidpathlist
         
         # target directory should not already exist:
         target = temporarydir.subdirectory('yodogg')
@@ -57,6 +62,9 @@ class TestFsFilesystem(object):
         # target and destination should be the same thing:
         assert target == destination
         
+        # files should be a valid path list:
+        assert isvalidpathlist(files)
+        
         # datadir, target and destination should each
         # contain the same amount of JPGs, regardless
         # of any of their structures:
@@ -73,6 +81,7 @@ class TestFsFilesystem(object):
     def test_flatten_with_new_suffix(self, datadir,
                                            temporarydir):
         from clu.testing.utils import countfiles
+        from clu.typology import isvalidpathlist
         
         # target directory should not already exist:
         target = temporarydir.subdirectory('yodogg')
@@ -81,28 +90,32 @@ class TestFsFilesystem(object):
         # and copying all files contained within it into
         # the target directory, renaming them as per their
         # original pathnames and giving *all* of them
-        # the “jpeg” suffix:
-        destination, files = datadir.flatten(target, new_suffix='jpeg')
+        # the “image” suffix:
+        destination, files = datadir.flatten(target, new_suffix='image')
         
         # target and destination should be the same thing:
         assert target == destination
         
+        # files should be a valid path list:
+        assert isvalidpathlist(files)
+        
         # datadir, target and destination should each
         # contain the same amount of things, regardless
         # of any of their structures or file suffixes:
         assert countfiles(datadir) \
-            == countfiles(target, suffix='jpeg') \
+            == countfiles(target, suffix='image') \
             == countfiles(destination) \
             == len(files)
         
         for f in files:
-            assert f.endswith('.jpeg')
+            assert f.endswith('.image')
             assert os.path.exists(f)
             assert os.path.basename(f) in destination
     
     def test_flatten_with_new_suffix_and_suffix_filter(self, datadir,
                                                              temporarydir):
         from clu.testing.utils import countfiles
+        from clu.typology import isvalidpathlist
         
         # target directory should not already exist:
         target = temporarydir.subdirectory('yodogg')
@@ -118,6 +131,9 @@ class TestFsFilesystem(object):
         # target and destination should be the same thing:
         assert target == destination
         
+        # files should be a valid path list:
+        assert isvalidpathlist(files)
+        
         # datadir, target and destination should each
         # contain the same amount of things, regardless
         # of any of their structures or file suffixes:
@@ -131,15 +147,34 @@ class TestFsFilesystem(object):
             assert os.path.exists(f)
             assert os.path.basename(f) in destination
     
-    def test_zip_archive(self, datadir):
-        from clu.fs.filesystem import temporary
-        from clu.typology import isvalidpathlist
+    def test_zip_archive_temporaryname(self, datadir, temporaryname):
+        from clu.testing.utils import countfiles
         
         # Ensure the “datadir” fixture has something
         # in it, of which we can make use:
-        datafiles = tuple(datadir.subpath(p) for p in datadir.ls_la())
-        assert len(datafiles) > 0
-        assert isvalidpathlist(datafiles)
+        assert countfiles(datadir) > 10
+        
+        # Use a TemporaryName instance for the zip archive:
+        tzip = temporaryname(prefix='test-zip-archive-',
+                             suffix='zip')
+        
+        fzip = datadir.zip_archive(tzip.name)
+        
+        assert tzip.exists
+        assert tzip.filesize > 10000
+        assert os.path.exists(tzip)
+        assert os.lstat(tzip).st_size > 10000
+        
+        assert os.path.exists(fzip)
+        assert os.path.samefile(tzip, fzip)
+    
+    def test_zip_archive(self, datadir):
+        from clu.fs.filesystem import temporary
+        from clu.testing.utils import countfiles
+        
+        # Ensure the “datadir” fixture has something
+        # in it, of which we can make use:
+        assert countfiles(datadir) > 10
         
         # Use a temporary file path for the zip archive:
         tzip = temporary(prefix='test-zip-archive-',
@@ -255,6 +290,7 @@ class TestFsFilesystem(object):
     def test_rm_rf(self, temporarydir):
         # Also involves `write_to_path(…)` and `Directory.walk()`
         from clu.fs.filesystem import write_to_path, rm_rf
+        from clu.testing.utils import countfiles
         
         # Make subdirectories:
         dd = temporarydir.subdirectory('yo')
@@ -286,24 +322,10 @@ class TestFsFilesystem(object):
         assert os.path.exists(ppp)
         assert os.path.exists(pppp)
         
-        yo_dogg_count = 0
-        
-        for root, dirs, files in dd.walk():
-            for filename in files:
-                if filename == 'yo-dogg.txt':
-                    yo_dogg_count += 1
-        
-        assert yo_dogg_count == 3
+        assert countfiles(dd, suffix='yo-dogg.txt') == 3
         assert rm_rf(dd)
         
-        yo_dogg_count = 0
-        
-        for root, dirs, files in dd.walk():
-            for filename in files:
-                if filename == 'yo-dogg.txt':
-                    yo_dogg_count += 1
-        
-        assert yo_dogg_count == 0
+        assert countfiles(dd, suffix='yo-dogg.txt') == 0
         
         # Re-check subdirectories:
         assert not dd.exists
@@ -541,10 +563,9 @@ class TestFsFilesystem(object):
     def test_hd(self):
         """ Tests for clu.fs.filesystem.hd """
         from clu.fs.filesystem import hd
+        from clu.fs.misc import gethomedir
         initial = os.getcwd()
         
-        gethomedir = lambda: os.path.expanduser("~")
-        
         with hd() as home:
             # print("* Testing directory-change instance: %s" % tmp.name)
             assert os.path.samefile(os.getcwd(),          gethomedir())
